package tacv.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.Set;

/**
 * Converts any type of a vector (or quaternion) to the desired one. 
 * The input and output class are supposed to have "x", "y", "z", ("w") fields.
 * 
 * It is also able to convert different coordination systems (RUF to RFU and vice versa).
 * 
 * @author Bc. Adam Horky
 *
 */
public class Converter {
	
	private Converter(){}
	
	public static <T> T convertVec(Object v1, Coords o1, T resObject, Coords o2){
		float x, y, z;
		Class<?> resClass = resObject.getClass();
		
		try{
			Class<?> clazz = v1.getClass();
			x = getValue(clazz, v1, "x");
			y = getValue(clazz, v1, "y");
			z = getValue(clazz, v1, "z");
			
			//convert x,y,z = x,z,-y and x,y,z = x,-z,y
			if(o1.equals(Coords.RFU)){
				if(o2.equals(Coords.RUF)){
					float tmp = z;
					z = -y;
					y = tmp;
				}
			} else if(o1.equals(Coords.RUF)){
				if(o2.equals(Coords.RFU)){
					float tmp = y;
					y = -z;
					z = tmp;
				}
			}
			
			setValue(resClass, resObject, "x", x);
			setValue(resClass, resObject, "y", y);
			setValue(resClass, resObject, "z", z);
			
			return resObject;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static <T> T convertVec(Object v1, T resObject){
		return convertVec(v1, Coords.DONT_CARE, resObject, Coords.DONT_CARE);
	}
	
	public static <T> T convertQuat(Object v1, Coords o1, T resObject, Coords o2){
		float w;
		Class<?> resClass = resObject.getClass();
		resObject = convertVec(v1, o1, resObject, o2);
		
		try{
			Class<?> clazz = v1.getClass();
			w = getValue(clazz, v1, "w");
			
			setValue(resClass, resObject, "w", w);
			
			return resObject;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static <T> T convertQuat(Object v1, T resObject){
		return convertQuat(v1, Coords.DONT_CARE, resObject, Coords.DONT_CARE);
	}
	
	private static void setValue(Class<?> resClass, Object resObject, String valueName, float value) throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException, NoSuchMethodException, InvocationTargetException{
		Field fx = findAtAllFields(resClass, valueName);
		fx.setAccessible(true);
		fx.set(resObject, value);
	}
	
	private static float getValue(Class<?> clazz, Object inObject, String valueName) throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException, InvocationTargetException, NoSuchMethodException{
		Field fx = findAtAllFields(clazz, valueName);
		fx.setAccessible(true);
		return ((Double)fx.getDouble(inObject)).floatValue();
	}
	
	private static Field findAtAllFields(Class<?> clazz, String fieldName) {
	    Field ret = findInClass(clazz, fieldName);
	    //all superclasses
	    if(ret == null){
			for (Class<?> superClass : getAllSupers(clazz)) {
		        ret = findInClass(superClass, fieldName);
		        if(ret != null){
		        	break;
		        }
		    }
	    }
	    return ret;
	}

	private static Set<Class<?>> getAllSupers(Class<?> clazz) {
	    Set<Class<?>> result = new HashSet<Class<?>>();
	    if (clazz != null) {
	        result.add(clazz);
	        result.addAll(getAllSupers(clazz.getSuperclass()));
//	        for (Class<?> interfaceClass : clazz.getInterfaces()) {
//	            result.addAll(getAllSupers(interfaceClass));
//	        }
	    }
	    return result;
	}
	
	private static Field findInClass(Class<?> superThing, String fieldName){
		for (Field f : superThing.getDeclaredFields()) {
            if(f.getName().equals(fieldName)){
            	return f;
            }
        }
		return null;
	}
	
    public enum Coords{
    	RUF, RFU,
    	DONT_CARE
    }

}
