package tacv.vis;

import cz.agents.alite.vis.Vis;
import cz.agents.alite.vis.element.Image;
import cz.agents.alite.vis.element.aggregation.ImageElements;
import cz.agents.alite.vis.element.implemetation.ImageImpl;
import cz.agents.alite.vis.layer.GroupLayer;
import cz.agents.alite.vis.layer.GroupVisLayer;
import cz.agents.alite.vis.layer.VisLayer;
import cz.agents.alite.vis.layer.terminal.ImageLayer;
import cz.agents.alite.vis.layer.toggle.KeyToggleLayer;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedHashMap;
import javax.imageio.ImageIO;
import javax.vecmath.Point3d;

public class BubbleLayer extends ImageLayer {

    final BubbleActivatorEntity bubbleActivatorEntity;
    final String bubbleText;
    static GroupVisLayer masterLayer;
    final long coolDown;
    long timeStamp;
    static final double pixelMovePerMillis = 0.05;

    private final double y;

    BubbleLayer(BubbleActivatorEntity spriteEntity, final BufferedImage image, String bubbleText, long coolDown) {
        super(new ImageElements() {

            @Override
            public Iterable<? extends Image> getImages() {
                return Arrays.asList(new ImageImpl(image));
            }

        });
        
        this.bubbleActivatorEntity = spriteEntity;
        this.bubbleText = bubbleText;
        timeStamp = System.currentTimeMillis();
        this.coolDown = coolDown;

        y = bubbleActivatorEntity.getPosition().y;
    }

    @Override
    public void paint(Graphics2D canvas) {
        long currentTimeMillis = System.currentTimeMillis();

        for (Image image : getImageElements().getImages()) {
            double x = bubbleActivatorEntity.getPosition().x;
            double yShift = -pixelMovePerMillis * (currentTimeMillis - timeStamp);
            AffineTransform t = AffineTransform.getTranslateInstance(Vis.transX(x) - image.getImage().getWidth() / 2.0, Vis.transY(y) + yShift - image.getImage().getHeight() / 2.0);
            canvas.drawImage(image.getImage(), t, null);

            canvas.setColor(Color.WHITE);
            canvas.drawString(bubbleText, (int) (Vis.transX(x) + image.getImage().getWidth() / 2.0), (int) (Vis.transY(y) + yShift + image.getImage().getHeight() / 2.0));

            if ((timeStamp + coolDown) < (currentTimeMillis)) {
                synchronized (BubbleLayer.class) {
                    masterLayer.removeSubLayer(this);
                }
            }
        }
    }

    @Override
    public String getLayerDescription() {
        String description = "Layer shows info bubbles showing selected entity actions.";
        return buildLayersDescription(description);
    }

    public static VisLayer create(BubbleActivatorEntity bubbleActivatorEntity, BufferedImage image, String bubbleText) {
        if (masterLayer == null) {
            masterLayer = KeyToggleLayer.create("a");

            VisLayer helpLayer = GroupLayer.create();
            helpLayer.setHelpOverrideString("[Actions] The layer shows important actions of the entities in form of bubble icons and title.");
            masterLayer.addSubLayer(helpLayer);
        }
        if (bubbleActivatorEntity != null) {
            VisLayer layer = new BubbleLayer(bubbleActivatorEntity, image, bubbleText, 1000);
            layer.setHelpOverrideString("");
            synchronized (BubbleLayer.class) {
                masterLayer.addSubLayer(layer);
            }
        }
        return masterLayer;
    }

    public static VisLayer create() {
        return create(null);
    }

    public static VisLayer create(BubbleActivatorEntity bubbleActivatorEntity) {
        return create(bubbleActivatorEntity, BubbleType.DEFAULT, "");
    }

    public static VisLayer create(BubbleActivatorEntity bubbleActivatorEntity, BubbleType type, String bubbleText) {
        return create(bubbleActivatorEntity, getBubble(type), bubbleText);
    }

    static LinkedHashMap<BubbleType, BufferedImage> bubbles = new LinkedHashMap<BubbleType, BufferedImage>();

    static BufferedImage getBubble(BubbleType type) {
        BufferedImage image = bubbles.get(type);
        if (image == null) {

            String fileName;
            switch (type) {
                case DONE: {
                    fileName = "resources/img/bubbles/done.png";
                    break;
                }
                case FAILED: {
                    fileName = "resources/img/bubbles/failed.png";
                    break;
                }
                case ERROR: {
                    fileName = "resources/img/bubbles/error.png";
                    break;
                }
                case BLOCADE: {
                    fileName = "resources/img/bubbles/blocade.png";
                    break;
                }
                case PLANNING: {
                    fileName = "resources/img/bubbles/planning.png";
                    break;
                }
                case STOP: {
                    fileName = "resources/img/bubbles/stop.png";
                    break;
                }
                case DEFAULT:
                default: {
                    fileName = "resources/img/bubbles/default.png";
                    break;
                }
            }
            try {
                image = ImageIO.read(new File(fileName));
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        return image;
    }

    public enum BubbleType {

        BLOCADE, PLANNING, STOP, DEFAULT, DONE, FAILED, ERROR

    }

    public static interface BubbleActivatorEntity {

        public Point3d getPosition();

    }
}
