package tacv.vis;

import java.awt.Color;
import java.util.LinkedList;

import org.jgrapht.Graph;
import org.jgrapht.graph.DefaultWeightedEdge;


import tacv.util.Point;
import cz.agents.alite.vis.element.Line;
import cz.agents.alite.vis.element.aggregation.LineElements;
import cz.agents.alite.vis.element.aggregation.PointElements;
import cz.agents.alite.vis.element.implemetation.LineImpl;
import cz.agents.alite.vis.layer.AbstractLayer;
import cz.agents.alite.vis.layer.GroupLayer;
import cz.agents.alite.vis.layer.VisLayer;
import cz.agents.alite.vis.layer.terminal.LineLayer;
import cz.agents.alite.vis.layer.terminal.PointLayer;
import tacv.universe.map.StreetGraph;

public class GraphLayer extends AbstractLayer {

    GraphLayer() {
    }

    static VisLayer create(final GraphEntity graphEntity, final Color edgeColor, final Color vertexColor,
            final int edgeStrokeWidth, final int vertexStrokeWidth) {
        GroupLayer group = GroupLayer.create();

        group.setHelpOverrideString("[Graph] This layer shows a graph.");
        

        // edges
        group.addSubLayer(LineLayer.create(new LineElements() {

            @Override
            public Iterable<Line> getLines() {
                LinkedList<Line> lines = new LinkedList<Line>();
                for (DefaultWeightedEdge edge : graphEntity.getGraph().edgeSet()) {
                    lines.add(new LineImpl(graphEntity.getGraph().getEdgeSource(edge), graphEntity.getGraph().getEdgeTarget(edge)));
                }
                return lines;
            }

            @Override
            public int getStrokeWidth() {
                return edgeStrokeWidth;
            }

            @Override
            public Color getColor() {
                return edgeColor;
            }

        }));

        // vertices
        group.addSubLayer(PointLayer.create(new PointElements() {

            @Override
            public Iterable<Point> getPoints() {
                LinkedList<Point> points = new LinkedList<Point>();
                for (Point vertex : graphEntity.getGraph().vertexSet()) {
                    points.add(vertex);
                }
                return points;
            }

            @Override
            public int getStrokeWidth() {
                return vertexStrokeWidth;
            }

            @Override
            public Color getColor() {
                return vertexColor;
            }

        }));

        return group;
    }

    public static VisLayer create(final StreetGraph streetGraph, final Color edgeColor, final Color vertexColor,
            final int edgeStrokeWidth, final int vertexStrokeWidth) {
        return create(new GraphEntity() {

            @Override
            public Graph<? extends Point, DefaultWeightedEdge> getGraph() {
                return streetGraph;
            }

        }, edgeColor, vertexColor, edgeStrokeWidth, vertexStrokeWidth);
    }

    private static interface GraphEntity {

        public Graph<? extends Point, DefaultWeightedEdge> getGraph();

    }

}
