package tacv.vis3d.spatial;

import com.jme3.asset.AssetManager;
import com.jme3.bounding.BoundingBox;
import com.jme3.math.FastMath;
import com.jme3.math.Quaternion;
import com.jme3.math.Vector3f;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.control.Control;
import cz.agents.alite.vis3d.Vis3d;

public final class TruckProvider {

    private static final Quaternion MODEL_LOAD_ROTATION = new Quaternion().fromAngles( -FastMath.HALF_PI, 0, 0);

    private static final String TRUCK_BODY_LOAD = "models/truck/truck.mesh.xml";

    private static final String TRUCK_WHEEL_LOAD = "models/truck/wheel.mesh.xml";

    private static TruckProvider instance;
    private Truck referentialInstance;

    Vis3d vis;

    private TruckProvider(Vis3d vis) {
        this.vis = vis;
        this.init();
    }

    public void init() {
        AssetManager assetManager = vis.getAssetManager();

        referentialInstance = new Truck("referentialTruck");

        Spatial body = assetManager.loadModel(TRUCK_BODY_LOAD);
        referentialInstance.attachChild(body);
        body.setLocalTranslation(new Vector3f(0, 0.3f, 0));
        body.setLocalRotation(MODEL_LOAD_ROTATION);
        body.setLocalScale(0.5f);

        Spatial wheelModel = assetManager.loadModel(TRUCK_WHEEL_LOAD);
        Node wheelLBB = new Node("LBB");
        wheelLBB.attachChild(wheelModel.deepClone());
        wheelLBB.setLocalTranslation(-0.75f, 0.8f, -2.3f);
        wheelLBB.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelLBB);

        Node wheelRBB = new Node("RBB");
        wheelRBB.attachChild(wheelModel.deepClone());
        wheelRBB.setLocalTranslation( 0.75f, 0.8f, -2.3f);
        wheelRBB.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelRBB);

        Node wheelLB = new Node("LB");
        wheelLB.attachChild(wheelModel.deepClone());
        wheelLB.setLocalTranslation(-0.75f, 0.8f, -1.3f);
        wheelLB.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelLB);

        Node wheelRB = new Node("RB");
        wheelRB.attachChild(wheelModel.deepClone());
        wheelRB.setLocalTranslation( 0.75f, 0.8f, -1.3f);
        wheelRB.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelRB);

        Node wheelLF = new Node("LF");
        wheelLF.attachChild(wheelModel.deepClone());
        wheelLF.setLocalTranslation(-0.75f, 0.8f, 1.85f);
        wheelLF.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelLF);

        Node wheelRF = new Node("RF");
        wheelRF.attachChild(wheelModel.deepClone());
        wheelRF.setLocalTranslation( 0.75f, 0.8f, 1.85f);
        wheelRF.setLocalScale(0.5f);
        referentialInstance.attachChild(wheelRF);

        referentialInstance.setModelBound(new BoundingBox());
        referentialInstance.updateModelBound();

    }

    public static TruckProvider getInstance(Vis3d vis){
        if(instance == null || instance.vis == null || !vis.equals(instance.vis)){
            instance = new TruckProvider(vis);
        }
        return instance;
    }

    public Node getNext(String name, Control ... ctrl){
        Node spatial = (Node)referentialInstance.deepClone();
        spatial.setName(name);

        for(Control c: ctrl){
            spatial.addControl(c);
        }

        return spatial;
    }
}
